# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
import sys

if __name__ == "__main__":
    from wasc.cli import wasc

    sys.exit(wasc())
